"""
Módulo com variáveis sobre a definição do driver
E extensões do driver.
"""

# Nome da pasta que ficará dentro da pasta temp.
# Nessa pasta serão armazenados logs e drivers do Selenium
TEMP_PATH_NAME = 'app_procuradoria'

# Url do GeckoDriver
URL_GECKODRIVER_WINDOWS = 'https://github.com/mozilla/geckodriver/releases/download/v0.32.0/geckodriver-v0.32.0-win64.zip'
URL_GECKODRIVER_LINUX = 'https://github.com/mozilla/geckodriver/releases/download/v0.35.0/geckodriver-v0.35.0-linux64.tar.gz'

# URLs da Extensão xPath
URL_ADDONS_XPATH = 'https://addons.mozilla.org/firefox/downloads/file/3588871/xpath_finder-1.0.2-fx.xpi'
